@@ -88,7 +88,6 @@ class Ensemble:
         num_jobs: int = 1,
         model: str = "fcnn",
         model_configs: Optional[Dict[str, Dict[str, Any]]] = None,
-        identifiers: Optional[Union[str, List[str]]] = None,
         predict_config: Optional[Dict[str, Any]] = None,
         sequential: Optional[bool] = None,
         temp_folder: str = "__tmp__",
@@ -100,7 +99,6 @@ class Ensemble:
             y,
             models=model,
             model_configs=model_configs,
-            identifiers=identifiers,
             predict_config=predict_config,
             sequential=sequential,
             num_jobs=num_jobs,
